//----------------------------------------------------------------------------
//
//  Core.nut
//
//  Should be loaded before all other scripts.
//
//----------------------------------------------------------------------------
IMAGE_PATH		<- "Images/"
SOUND_PATH		<- ""
SCRIPT_PATH		<- "Scripts/"
DEBUG			<- true

function debug( x )
{
	if ( DEBUG )
		print( x );
}

function PrintException( exception, ... )
{
	local text = "Squirrel Exception: '" + exception + "'";	
	if ( vargc > 0 ) text += "\n\n" + vargv[0];
	print( text );
}

function GetRealPath( path )
{
	if ( path.slice(-4) != ".nut" )
		path += ".nut";
	
	return SCRIPT_PATH + path;
}

function compile( path )
{
	path = GetRealPath( path );
	debug( "Compiling: " + path );
	return loadfile( path );
}

function load( path )
{
	path = GetRealPath( path );
	debug( "Loading: " + path );
	
	try
	{
		dofile( path, true );
	}
	catch( e )
	{
		PrintException( e, "Could not load file '" + path + "'" );
		Error( "Failed to load script file: " + path );
		return false;
	}
	
	return true;
}

function run( path, ... )
{
	local channel;
	if ( vargc==1 ) channel = vargv[0];
	else channel = 0;
	
	debug( "Running script: " + path )
	Script.RunScript( GetRealPath(path), channel );
}

function sleep( time )
{
	Script.Sleep( time );
	suspend();
}

function done()
{
	Script.EndScript();
}

function SaveConfig( path )
{
	local f = file( path, "w" )
	debug( "Saving config: " + path )
	Write( f, config )
}

function Write( f, data )
{
	switch ( type(data) )
	{
		case "instance":
			f.writestr( data.tostring() );
			break;
		
		case "integer":
			f.writestr( format("%d", data) );
			break;
			
		case "float":
			f.writestr( format("%f", data) );
			break;
			
		case "array":
			f.writestr( "[" );
			local i;
			for ( i=0; i<data.len()-1; i++ )
			{
				Write( f, data[i] );
				f.writestr( ", " );
			}
			Write( f, data[data.len()-1] );
			f.writestr( "]" );
			break;
			
		case "table":
			f.writestr( "{\n\t" );
			foreach ( slot in data )
			{
				Write( f, slot );
				f.writestr( ";\n" );
			}
			f.writestr( "}" );
			break;
		
		case "string":
			f.writestr( data );
			break;
		
		case "bool":
			if ( data ) f.writestr( "true" );
			else f.writestr( "false" );
			break;
		
		case "null":
			f.writestr( "null" );
			break;
			
		default:
			throw ( "Unknown type, cannot write to file: " + type(data) );
	}
}

indent <- -1;

function _print( data )
{
	local str = "";
	
	for ( local i=0; i<indent; i++ )
	{
		str += "    ";
	}
	
	switch ( type(data) )
	{
		case "integer":
		case "string":
		case "float":
			print( str + data );
			break;
		
		case "bool":
			if(data) print(str + "true");
			else print(str + "false");
			break;
			
		case "null":
			print( str + "null" );
			break;
		
		default:
			print( str + type(data) );
	}
	
	//print( str + data._tostring() );
}

function PrintObject( data )
{
	indent++;

	switch ( type(data) )
	{
		case "instance":
			_print( "instance" );
			break;
		
		case "integer":
			_print( format("%d", data) );
			break;
			
		case "float":
			_print( format("%f", data) );
			break;
			
		case "array":
			_print( "[" );
			local i;
			for ( i=0; i<data.len()-1; i++ )
			{
				PrintObject( data[i] );
				_print( ", " );
			}
			PrintObject( data[data.len()-1] );
			_print( "]" );
			break;
			
		case "table":
			_print( "{" );
			foreach ( slot in data )
			{
				PrintObject( slot );
				_print( ";\n" );
			}
			_print( "}" );
			break;
		
		case "string":
			_print( data );
			break;
		
		case "bool":
			if ( data ) _print( "true" );
			else _print( "false" );
			break;
		
		case "null":
			_print( "null" );
			break;
			
		case "class":
			_print( "class" );
			break;
			
		case "function":
			_print( "function" );
			break;
			
		default:
			_print( data ); /* ( "Unknown type, cannot print: " + type(data) );*/
	}
	
	indent--;
}

function PrintObj( o )
{
	local t = type(o);
	
	if ( t=="class" || t=="function" )
		return;
	
	if ( indent==1 && (t!="instance" && t!="array") )
		return;
	
	numGlobals++;
	
	if ( t=="table" || t=="array" )
	{
		_print( t );
		_print( "{" );
		indent++;
		
		if ( o.len() > 0 )
		{
			foreach ( x in o )
			{
				PrintObj( x );
			}
		}
		else
		{
			_print( o );
		}
		
		indent--;
		_print( "}" );
		_print( "" );
	}
	else
	{
		_print( o );
	}
}

function PrintGlobals()
{
	local root = getroottable();
	
	numGlobals = 0;
	
	foreach ( x in root )
	{
		if ( type(x)=="instance" || type(x)=="array" )
		{
			numGlobals++;
			print( x );
		}
	}
	
	print( "\nNumGlobals: " + numGlobals );
}

